@@ -1,4 +1,68 @@
-makeS <- function(pedigree, heterogametic, DosageComp = c(NULL, "ngdc", "hori", "hedo", "hoha", "hopi"), returnS = FALSE){
+#' Creates the additive genetic relationship matrix for the shared sex
+#' chromosomes
+#' 
+#' The function returns the inverse of the additive relationship matrix in
+#' sparse matrix format for the sex chromosomes (e.g., either X or Z).
+#' 
+#' Missing parents (e.g., base population) should be denoted by either 'NA',
+#' '0', or '*'.
+#' 
+#' The inverse of the sex-chromosome additive genetic relationship matrix
+#' (S-matrix) is constructed implementing the Meuwissen and Luo (1992)
+#' algorithm to directly construct inverse additive relationship matrices
+#' (borrowing code from Jarrod Hadfield's MCMCglmm function, \code{inverseA})
+#' and using equations presented in Fernando & Grossman (1990; see Wolak et al.
+#' 2013).  Additionally, the S-matrix itself can be constructed (although this
+#' takes much longer than computing S-inverse directly).
+#' 
+#' The choices of dosage compensation models are: no global dosage compensation
+#' ("ngdc"), random inactivation in the homogametic sex ("hori"), doubling of
+#' the single shared sex chromosome in the heterogametic sex ("hedo"), halving
+#' expression of both sex chromosomes in the homogametic sex ("hoha"), or
+#' inactivation of the paternal sex chromosome in the homogametic sex ("hopi").
+#' 
+#' @param pedigree A pedigree where the columns are ordered ID, Dam, Sire, Sex
+#' @param heterogametic Character indicating the label corresponding to the
+#'   heterogametic sex used in the \dQuote{Sex} column of the pedigree
+#' @param DosageComp A character indicating which model of dosage compensation.
+#'   If \code{NULL} then the \dQuote{ngdc} model is assumed.
+#' @param returnS Logical statement, indicating if the relationship matrix
+#'   should be constructed in addition to the inverse
+#'
+#' @return a \code{list}:
+#'   \describe{
+#'     \item{model }{the model of sex-chromosome dosage compensation assumed.}
+#'     \item{S }{the sex-chromosome relationship matrix in sparse matrix
+#'       form or NULL if \code{returnS} = FALSE}
+#'     \item{logDet }{the log determinant of the S matrix}
+#'     \item{Sinv }{the inverse of the S matrix in sparse matrix form}
+#'     \item{listSinv }{the three column form of the non-zero elements for the 
+#'       inverse of the S matrix}
+#'     \item{inbreeding }{the sex-linked inbreeding coefficients for all 
+#'       individuals in the pedigree}
+#'     \item{vii }{a vector of the (non-zero) elements of the diagonal V matrix
+#'       of the S=TVT' decomposition. Contains the variance of Mendelian
+#'       sampling for a sex-linked locus}
+#'   }
+#' @author \email{matthewwolak@@gmail.com}
+#' @references Wolak, M.E., D.A. Roff, and D.J. Fairbairn. in prep. The
+#' contribution of sex chromosomal additive genetic (co)variation to the
+#' phenotypic resemblance between relatives under alternative models of dosage
+#' compensation.
+#' 
+#' Fernando, R.L. & Grossman, M. 1990. Genetic evaluation with autosomal and
+#' X-chromosomal inheritance. Theoretical and Applied Genetics, 80:75-80.
+#' 
+#' Meuwissen, T.H.E. and Z. Luo. 1992. Computing inbreeding coefficients in
+#' large populations. Genetics, Selection, Evolution, 24:305-313.
+#' @examples
+#' 
+#'  makeS(FG90, heterogametic = "0", returnS = TRUE)
+#' 
+#' @export
+makeS <- function(pedigree, heterogametic,
+	DosageComp = c(NULL, "ngdc", "hori", "hedo", "hoha", "hopi"),
+	returnS = FALSE){
 
     if(length(unique(pedigree[,4])) > 2) stop("Error: more than 2 sexes specified")
     nPed <- numPed(pedigree[, 1:3])
@@ -45,7 +109,7 @@ makeS <- function(pedigree, heterogametic, DosageComp = c(NULL, "ngdc", "hori",
           Vii <- (sex + 1)/2
           f <- c(rep(0, N), -1)
 
-          Cout <- .C("sinv",
+          Cout <- .C("sinv", PACKAGE = "nadiv",
 	    as.integer(nPed[, 2] - 1), #dam
 	    as.integer(nPed[, 3] - 1),  #sire
 	    as.double(f),  #f
@@ -94,7 +158,7 @@ makeS <- function(pedigree, heterogametic, DosageComp = c(NULL, "ngdc", "hori",
              Vii <- (2 - sex)
              f <- c(rep(0, N), -1)
 
-             Cout <- .C("sinv",
+             Cout <- .C("sinv", PACKAGE = "nadiv",
 	       as.integer(nPed[, 2] - 1), #dam
 	       as.integer(nPed[, 3] - 1),  #sire
 	       as.double(f),  #f
@@ -138,7 +202,7 @@ makeS <- function(pedigree, heterogametic, DosageComp = c(NULL, "ngdc", "hori",
                 Vii <- rep(1, N) 
                 f <- rep(0, N)
 
-             Cout <- .C("sinv",
+             Cout <- .C("sinv", PACKAGE = "nadiv",
 	       as.integer(nPed[, 2] - 1), #dam
 	       as.integer(nPed[, 3] - 1),  #sire
 	       as.double(f),  #f
@@ -166,17 +230,20 @@ makeS <- function(pedigree, heterogametic, DosageComp = c(NULL, "ngdc", "hori",
            }
       }              
 
-
+    logDet <- -1*determinant(Sinv, logarithm = TRUE)$modulus[1]
     listSinv <- sm2list(Sinv, rownames = as.character(pedigree[, 1]), colnames = c("Row", "Column", "Sinverse"))
     if(returnS){
        cat("S-inverse made: Starting to make S...")
           T <- as(solve(Q), "dgCMatrix")
           S <- as(t(T) %*% Diagonal(N, Vii) %*% T, "dgCMatrix")
+            S@Dimnames <- list(as.character(pedigree[, 1]), NULL)
        cat(".done", "\n")
     } else{
          S <- NULL
       }
 
-return(list(model = dc.model, S = S, Sinv = Sinv, listSinv = listSinv, inbreeding = f, v = Vii))
+return(list(model = dc.model, S = S, logDet = logDet,
+	Sinv = Sinv, listSinv = listSinv,
+	inbreeding = f, vii = Vii))
 }
 
